#ifndef NXTDEFS__H
#define NXTDEFS__H 1
//==============================================================================
//
// Copyright (C) 2006 - John Hansen. All rights reserved.
//
// Workfile:: NXTDefs.h
// Date:: 2006-12-13
// Revision:: 13
//
//------------------------------------------------------------------------------
//
// Contains declarations for the NBC NXT API resources
//
//==============================================================================

#include "NBCCommon.h"

// define structures for various system calls

dseg	segment

TLocation	struct
 X		sword
 Y		sword
TLocation	ends

TSize	struct
 Width	sword
 Height	sword
TSize	ends

// FileOpenRead, FileOpenWrite, FileOpenAppend
TFileOpen	struct
 Result		word
 FileHandle	byte
 Filename	byte[]
 Length		dword
TFileOpen	ends

// FileRead, FileWrite
TFileReadWrite	struct
 Result		word
 FileHandle	byte
 Buffer		byte[]
 Length		dword
TFileReadWrite	ends

// FileClose
TFileClose	struct
 Result		word
 FileHandle	byte
TFileClose	ends

// FileResolveHandle
TFileResolveHandle	struct
 Result		word
 FileHandle	byte
 WriteHandle	byte
 Filename	byte[]
TFileResolveHandle	ends

// FileRename
TFileRename	struct
 Result		word
 OldFilename	byte[]
 NewFilename	byte[]
TFileRename	ends

// FileDelete
TFileDelete	struct
 Result		word
 Filename	byte[]
TFileDelete	ends

// SoundPlayFile
TSoundPlayFile	struct
 Result		sbyte
 Filename	byte[]
 Loop		byte
 Volume		byte
TSoundPlayFile	ends

// SoundPlayTone
TSoundPlayTone	struct
 Result		sbyte
 Frequency	word
 Duration	word
 Loop		byte
 Volume		byte
TSoundPlayTone	ends

// SoundGetState
TSoundGetState	struct
 State		byte
 Flags		byte
TSoundGetState	ends

// SoundSetState
TSoundSetState	struct
 Result		byte
 State		byte
 Flags		byte
TSoundSetState	ends

// DrawText
TDrawText	struct
 Result		sbyte
 Location	TLocation
 Text		byte[]
 Options	dword
TDrawText	ends

// DrawPoint
TDrawPoint	struct
 Result		sbyte
 Location	TLocation
 Options	dword
TDrawPoint	ends

// DrawLine
TDrawLine	struct
 Result		sbyte
 StartLoc	TLocation
 EndLoc		TLocation
 Options	dword
TDrawLine	ends

// DrawCircle
TDrawCircle	struct
 Result		sbyte
 Center		TLocation
 Size		byte
 Options	dword
TDrawCircle	ends

// DrawRect
TDrawRect	struct
 Result		sbyte
 Location	TLocation
 Size		TSize
 Options	dword
TDrawRect	ends

// DrawGraphic
TDrawGraphic	struct
 Result		sbyte
 Location	TLocation
 Filename	byte[]
 Variables	sdword[]
 Options	dword
TDrawGraphic	ends

// SetScreenMode
TSetScreenMode	struct
 Result		sbyte
 ScreenMode	dword
TSetScreenMode	ends

// ReadButton
TReadButton	struct
 Result		sbyte
 Index		byte
 Pressed	byte
 Count		byte
 Reset		byte
TReadButton	ends

// CommLSWrite
TCommLSWrite	struct
 Result		sbyte
 Port		byte
 Buffer		byte[]
 ReturnLen	byte
TCommLSWrite	ends

// CommLSRead
TCommLSRead	struct
 Result		sbyte
 Port		byte
 Buffer		byte[]
 BufferLen	byte
TCommLSRead	ends

// CommLSCheckStatus
TCommLSCheckStatus	struct
 Result		sbyte
 Port		byte
 BytesReady	byte
TCommLSCheckStatus	ends

// RandomNumber
TRandomNumber	struct
 Result		sword
TRandomNumber	ends

// GetStartTick
TGetStartTick	struct
 Result		dword
TGetStartTick	ends

// MessageWrite
TMessageWrite	struct
 Result		sbyte
 QueueID	byte
 Message	byte[]
TMessageWrite	ends

// MessageRead
TMessageRead	struct
 Result		sbyte
 QueueID	byte
 Remove		byte
 Message	byte[]
TMessageRead	ends

// CommBTCheckStatus
TCommBTCheckStatus	struct
 Result		sbyte
 Connection	byte
TCommBTCheckStatus	ends

// CommBTWrite
TCommBTWrite	struct
 Result		sbyte
 Connection	byte
 Buffer		byte[]
TCommBTWrite	ends

// CommBTRead
TCommBTRead	struct
 Result		sbyte
 Count		byte
 Buffer		byte[]
TCommBTRead	ends

// KeepAlive
TKeepAlive	struct
 Result		dword
TKeepAlive	ends

// IOMapRead
TIOMapRead	struct
 Result		sbyte
 ModuleName	byte[]
 Offset		word
 Count		word
 Buffer		byte[]
TIOMapRead	ends

// IOMapWrite
TIOMapWrite	struct
 Result		sbyte
 ModuleName	byte[]
 Offset		word
 Buffer		byte[]
TIOMapWrite	ends

dseg	ends

// motor arrays (compiler will optimize these out if they are not used)
dseg segment
  OUT_AB byte[] OUT_A, OUT_B
  OUT_AC byte[] OUT_A, OUT_C
  OUT_BC byte[] OUT_B, OUT_C
  OUT_ABC byte[] OUT_A, OUT_B, OUT_C
  __OnRev_Tmp sbyte
dseg ends

// API macros

#define OnFwd(ports, pwr) setout ports, Power, pwr, OutputMode, OUT_MODE_MOTORON+OUT_MODE_BRAKE, RunState, OUT_RUNSTATE_RUNNING, TachoLimit, 0, UpdateFlags, UF_UPDATE_SPEED+UF_UPDATE_MODE+UF_UPDATE_TACHO_LIMIT
#define OnRev(ports, pwr) \
  neg __OnRev_Tmp, pwr \
  OnFwd(ports, __OnRev_Tmp)

#define Coast(ports) setout ports, Power, 0, OutputMode, OUT_MODE_BRAKE, RegMode, OUT_REGMODE_IDLE, RunState, OUT_RUNSTATE_IDLE, UpdateFlags, UF_UPDATE_SPEED+UF_UPDATE_MODE
#define Off(ports) setout ports, Power, 0, OutputMode, OUT_MODE_MOTORON+OUT_MODE_BRAKE, RunState, OUT_RUNSTATE_RUNNING, UpdateFlags, UF_UPDATE_SPEED+UF_UPDATE_MODE

#define OnFwdReg(ports, pwr, regmode) setout ports, Power, pwr, OutputMode, OUT_MODE_MOTORON+OUT_MODE_REGULATED+OUT_MODE_BRAKE, RegMode, regmode, RunState, OUT_RUNSTATE_RUNNING, TachoLimit, 0, UpdateFlags, UF_UPDATE_SPEED+UF_UPDATE_MODE+UF_UPDATE_TACHO_LIMIT
#define OnRevReg(ports, pwr, regmode) \
  neg __OnRev_Tmp, pwr \
  OnFwdReg(ports, __OnRev_Tmp, regmode)

#define OnFwdSync(ports, pwr, turnpct) setout ports, Power, pwr, OutputMode, OUT_MODE_MOTORON+OUT_MODE_REGULATED+OUT_MODE_BRAKE, RegMode, OUT_REGMODE_SYNC, TurnRatio, turnpct, RunState, OUT_RUNSTATE_RUNNING, TachoLimit, 0, UpdateFlags, UF_UPDATE_SPEED+UF_UPDATE_MODE+UF_UPDATE_TACHO_LIMIT
#define OnRevSync(ports, pwr, turnpct) \
  neg __OnRev_Tmp, pwr \
  OnFwdSync(ports, __OnRev_Tmp, turnpct)

#define RotateMotor(ports, pwr, angle) \
  arrbuild __rotate_ports, ports \
  mov __rotate_power, pwr \
  mov __rotate_angle, angle \
  set __rotate_sync, FALSE \
  call __RotateMotor

#define RotateMotorEx(ports, pwr, angle, turnpct, bSync) \
  arrbuild __rotate_ports, ports \
  mov __rotate_power, pwr \
  mov __rotate_angle, angle \
  mov __rotate_turnpct, turnpct \
  mov __rotate_sync, bSync \
  call __RotateMotor

subroutine __RotateMotor
// code from mot7.nbc (modified)
dseg segment
  __rotate_power byte
  __rotate_angle slong
  __rotate_ports byte[]
  __rotate_firstPort byte
  __rotate_sync byte
  __rotate_turnpct sbyte
  __rotate_theUF byte
  __rotate_theOM byte
  __rotate_theRM byte
  __rotate_theRS byte
// Note these PID values may not be optimal, someone with more knowledge of PID
// algorithms should probably work on them
  __rotate_theRVP byte 40
  __rotate_theRVI byte 20
  __rotate_theRVD byte 100
  __rotate_rs byte
  __rotate_OldRotCount sword
  __rotate_RotCount sword
  __rotate_thePower sbyte
  __rotate_theAngle ulong
  __rotate_theTurnPct sbyte
dseg ends
  // __rotate_power, __rotate_angle, and __rotate_ports have been set already
  sign __rotate_thePower, __rotate_angle
  abs __rotate_theAngle, __rotate_angle
  mul __rotate_thePower, __rotate_thePower, __rotate_power // convert __rotate_power to negative value if __rotate_angle is negative.

  brtst EQ, __rotate_NoSync, __rotate_sync
  set __rotate_theOM, OUT_MODE_MOTORON+OUT_MODE_BRAKE+OUT_MODE_REGULATED
  set __rotate_theRM, OUT_REGMODE_SYNC
  mov __rotate_theTurnPct, __rotate_turnpct
  jmp __rotate_Start
__rotate_NoSync:
  set __rotate_theOM, OUT_MODE_MOTORON+OUT_MODE_BRAKE
  set __rotate_theRM, OUT_REGMODE_IDLE
  set __rotate_theTurnPct, 0
__rotate_Start:
  set __rotate_theRS, OUT_RUNSTATE_RUNNING
  set __rotate_theUF, UF_UPDATE_RESET_BLOCK_COUNT+UF_UPDATE_RESET_COUNT+UF_UPDATE_TACHO_LIMIT+UF_UPDATE_SPEED+UF_UPDATE_MODE+UF_UPDATE_PID_VALUES
  setout __rotate_ports, OutputMode, __rotate_theOM, RegMode, __rotate_theRM, TachoLimit, __rotate_theAngle, RunState, __rotate_theRS, RegPValue, __rotate_theRVP, RegIValue, __rotate_theRVI, RegDValue, __rotate_theRVD, Power, __rotate_thePower, TurnRatio, __rotate_turnpct, UpdateFlags, __rotate_theUF

// Waits till angle reached
  index __rotate_firstPort, __rotate_ports, NA
__rotate_Running:
  getout __rotate_rs, __rotate_firstPort, RunState
  brcmp EQ, __rotate_Running, __rotate_rs, OUT_RUNSTATE_RUNNING
// Regulates for speed = 0
  set __rotate_theOM, OUT_MODE_MOTORON+OUT_MODE_BRAKE+OUT_MODE_REGULATED
  set __rotate_theRM, OUT_REGMODE_SPEED
  set __rotate_theUF, UF_UPDATE_SPEED+UF_UPDATE_MODE
  setout __rotate_ports, OutputMode, __rotate_theOM, RegMode, __rotate_theRM, RunState, __rotate_theRS, Power, 0, UpdateFlags, __rotate_theUF
// Verifies that motor doesn't rotate for 50ms, else loops
  getout __rotate_RotCount, __rotate_firstPort, RotationCount
__rotate_Stabilize:
  mov __rotate_OldRotCount, __rotate_RotCount
  wait 50
  getout __rotate_RotCount, __rotate_firstPort, RotationCount
  brcmp NEQ, __rotate_Stabilize, __rotate_OldRotCount, __rotate_RotCount
  return
ends

dseg segment
  __SensorInvalidTmp byte
dseg ends

#define SetSensorType(port,type) set InputIOType(port), type
#define SetSensorMode(port,mode) set InputIOInputMode(port), mode
#define ReadSensor(port,value) mov value, InputIOScaledValue(port)
#define ClearSensor(port) set InputIOScaledValue(port), 0

#define SetSensorTouch(port) \
  SetSensorType(port,IN_TYPE_SWITCH) \
  SetSensorMode(port,IN_MODE_BOOLEAN) \
  ResetSensor(port)

#define SetSensorLight(port) \
  SetSensorType(port,IN_TYPE_LIGHT_ACTIVE) \
  SetSensorMode(port,IN_MODE_PCTFULLSCALE) \
  ResetSensor(port)

#define SetSensorSound(port) \
  SetSensorType(port,IN_TYPE_SOUND_DB) \
  SetSensorMode(port,IN_MODE_PCTFULLSCALE) \
  ResetSensor(port)

#define SetSensorUltrasonic(port) \
  SetSensorType(port,IN_TYPE_LOWSPEED_9V) \
  SetSensorMode(port,IN_MODE_RAW) \
  ResetSensor(port)

dseg segment
  __ResetSensorMutex mutex
  __ResetSensorPort byte
  __ResetSensorTmp byte
dseg ends

subroutine __ResetSensor
  setin TRUE, __ResetSensorPort, InvalidData
__SensorStillInvalid:
  getin	__ResetSensorTmp, __ResetSensorPort, InvalidData
  brtst	NEQ, __SensorStillInvalid, __ResetSensorTmp
  return
ends

#define ResetSensor(port) \
  acquire __ResetSensorMutex \
  mov __ResetSensorPort, port \
  call __ResetSensor \
  release __ResetSensorMutex

; US Sensor read routine
; Based on ReadUsSensor by David Wallace

#define ReadSensorUS(port,value) \
  acquire __us_i2c_mutex \
  mov __us_i2c_wArgs.Port, port \
  call __ReadSensorUS \
  mov value, __us_RawDist \
  release __us_i2c_mutex

subroutine __ReadSensorUS

dseg segment
  __us_i2c_mutex mutex
  __us_i2c_wArgs TCommLSWrite
  __us_i2c_cmd_bufLSWrite1 byte[] 0x2, 0x42
  __us_i2c_rArgs TCommLSRead
  __us_i2c_cmd_readBuf byte[] 0x21
  __us_i2c_BytesRead byte
  __us_i2c_csArgs TCommLSCheckStatus
  __us_readUsThen dword
  __us_readUsNow dword
  __us_i2c_FailedWrite dword
  __us_i2c_FailedStatus dword
  __us_RawDist byte
dseg ends

  ; Setup US sensor for reading
  set __us_i2c_wArgs.ReturnLen, 1
  mov __us_i2c_wArgs.Buffer, __us_i2c_cmd_bufLSWrite1
  syscall CommLSWrite, __us_i2c_wArgs
  brtst EQ, lbl_i2cwOk, __us_i2c_wArgs.Result
  add __us_i2c_FailedWrite, __us_i2c_FailedWrite, 1 ; increment failed write counter
                                        ; print if bad sensor suspected
  jmp lbl_readUsEnd

lbl_i2cwOk:
  ; Check US sensor status
  mov __us_i2c_csArgs.Port, __us_i2c_wArgs.Port

  gettick __us_readUsNow
  add __us_readUsThen, __us_readUsNow, 100 ; Limit loop to 100 ms in case check status fails
lbl_checkI2cStatus:
  syscall CommLSCheckStatus, __us_i2c_csArgs
  gettick __us_readUsNow
  brcmp GT, lbl_i2c_FailedStatus, __us_readUsNow, __us_readUsThen ; Sometimes status check is never successful! So, must limit loop.
  brtst NEQ, lbl_checkI2cStatus, __us_i2c_csArgs.Result ; Result must be 0 for successful transaction
  brtst EQ, lbl_i2c_FailedStatus, __us_i2c_csArgs.BytesReady ; Must have bytes to read to continue

  ; Read US sensor
  mov __us_i2c_rArgs.Port, __us_i2c_wArgs.Port
  set __us_i2c_rArgs.BufferLen, 1
  syscall CommLSRead, __us_i2c_rArgs
  mov __us_i2c_cmd_readBuf, __us_i2c_rArgs.Buffer
  arrsize __us_i2c_BytesRead, __us_i2c_cmd_readBuf
  brtst LTEQ, lbl_readUsEnd, __us_i2c_BytesRead
  index __us_RawDist, __us_i2c_cmd_readBuf, 0
  ; Observations:
  ;   Seems to return approximate distance in centimetres.
  ;   I see readings of 255 come back frequently - especially when movement is detected.

lbl_i2c_FailedStatus:
  add __us_i2c_FailedStatus, __us_i2c_FailedStatus, 1 ; increment failed status counter
                                          ; print if bad sensor suspected
lbl_readUsEnd:
  return
ends

dseg segment
  __TextOutMutex mutex
  __TextOutArgs TDrawText
dseg ends

#define TextOut(x,y,cls,txt) \
  acquire __TextOutMutex \
  mov __TextOutArgs.Location.X,x \
  mov __TextOutArgs.Location.Y,y \
  mov __TextOutArgs.Options,cls \
  mov __TextOutArgs.Text,txt \
  syscall DrawText,__TextOutArgs \
  release __TextOutMutex

#define NumOut(x,y,cls,num) \
  acquire __TextOutMutex \
  mov __TextOutArgs.Location.X,x \
  mov __TextOutArgs.Location.Y,y \
  mov __TextOutArgs.Options,cls \
  numtostr __TextOutArgs.Text,num \
  syscall DrawText,__TextOutArgs \
  release __TextOutMutex

dseg segment
  __PlayToneTmp TSoundPlayTone
  __PlayFileTmp TSoundPlayFile
dseg ends

#define PlayTone(freq,dur) \
  mov __PlayToneTmp.Frequency, freq \
  mov __PlayToneTmp.Duration, dur \
  set __PlayToneTmp.Volume, 4 \
  set __PlayToneTmp.Loop, 0 \
  syscall SoundPlayTone, __PlayToneTmp

#define PlayFile(file) \
  mov __PlayFileTmp.Filename, file \
  set __PlayFileTmp.Volume, 4 \
  set __PlayFileTmp.Loop, 0 \
  syscall SoundPlayFile, __PlayFileTmp

dseg segment
   __RandomTmp dword
   __RandomArgs TRandomNumber
   __RandomMutex mutex
dseg ends

#define Random(arg,max) \
   acquire __RandomMutex \
   syscall RandomNumber, __RandomArgs \
   mov __RandomTmp, __RandomArgs.Result \
   add __RandomTmp, __RandomTmp, 32768 \
   mul __RandomTmp, __RandomTmp, max \
   div __RandomTmp, __RandomTmp, 65536 \
   mov arg, __RandomTmp \
   release __RandomMutex

dseg segment
   __KeepAliveArgs TKeepAlive
dseg ends

#define ResetSleepTimer syscall KeepAlive, __KeepAliveArgs


#endif // NXTDEFS__H
